var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/channel-mock.ts
import { TextChannel, ChannelType, ForumChannel, ThreadChannel, Message as Message2, Invite, Collection, SnowflakeUtil, NewsChannel, MessageReaction, DiscordAPIError, MessagePayload as MessagePayload2 } from "discord.js";
import { isSnowflakeLarger, isSnowflakeLargerAsInt, randomSnowflake as randomSnowflake4, sortMessagesById } from "@answeroverflow/discordjs-utils";

// src/guild-mock.ts
import { PermissionsBitField as PermissionsBitField2, Guild, Role } from "discord.js";
import { randomSnowflake as randomSnowflake2 } from "@answeroverflow/discordjs-utils";

// src/user-mock.ts
import { ClientUser, GuildMember, PermissionsBitField, User } from "discord.js";
import { randomSnowflake } from "@answeroverflow/discordjs-utils";
function mockUser(client, data = {}) {
  const rawData = {
    id: randomSnowflake().toString(),
    username: "USERNAME",
    discriminator: "user#0000",
    avatar: "user avatar url",
    bot: false,
    ...data
  };
  const user = Reflect.construct(User, [
    client,
    rawData
  ]);
  client.users.cache.set(user.id, user);
  return user;
}
__name(mockUser, "mockUser");
function mockClientUser(client, override = {}) {
  const rawData = {
    id: process.env.DISCORD_CLIENT_ID ?? process.env.VITEST_DISCORD_CLIENT_ID ?? randomSnowflake().toString(),
    username: "test",
    discriminator: "0000",
    avatar: null,
    bot: false,
    ...override
  };
  const clientUser = Reflect.construct(ClientUser, [
    client,
    rawData
  ]);
  client.user = clientUser;
  client.user.id = rawData.id;
  return clientUser;
}
__name(mockClientUser, "mockClientUser");
function mockGuildMember(input) {
  const { client, permissions = PermissionsBitField.Default, data = {} } = input;
  let { user, guild } = input;
  if (!user) {
    user = mockUser(client);
  }
  if (!guild) {
    guild = mockGuild(client, user);
  }
  const role = mockRole(client, permissions, guild);
  const rawData = {
    guild_id: guild.id,
    roles: [
      role.id
    ],
    deaf: false,
    user: {
      id: user.id
    },
    joined_at: "33",
    mute: false,
    ...data
  };
  const member = Reflect.construct(GuildMember, [
    client,
    rawData,
    guild
  ]);
  guild.members.cache.set(member.id, member);
  return member;
}
__name(mockGuildMember, "mockGuildMember");

// src/guild-mock.ts
import { omit } from "@answeroverflow/utils";
function mockGuild(client, owner, data = {}) {
  if (!owner) {
    owner = mockUser(client);
  }
  const guildId = data.id ?? randomSnowflake2().toString();
  const rawData = {
    id: guildId,
    owner_id: owner.id,
    verification_level: 0,
    emojis: [],
    icon: "guild icon url",
    mfa_level: 0,
    hub_type: 0,
    features: [],
    roles: [],
    name: "guild name",
    description: "guild description",
    default_message_notifications: 0,
    banner: "guild banner url",
    splash: "guild splash url",
    discovery_splash: "guild discovery splash url",
    region: "",
    afk_channel_id: null,
    afk_timeout: 60,
    explicit_content_filter: 0,
    application_id: null,
    system_channel_id: null,
    system_channel_flags: 0,
    rules_channel_id: null,
    vanity_url_code: null,
    premium_tier: 0,
    preferred_locale: "",
    public_updates_channel_id: null,
    nsfw_level: 0,
    stickers: [],
    premium_progress_bar_enabled: false,
    ...omit(data, "id")
  };
  const guild = Reflect.construct(Guild, [
    client,
    rawData
  ]);
  mockRole(client, PermissionsBitField2.Default, guild, {
    name: "everyone",
    id: guild.id
  });
  client.guilds.cache.set(guild.id, guild);
  mockGuildMember({
    client,
    user: owner,
    guild
  });
  mockGuildMember({
    client,
    user: client.user,
    guild
  });
  guild.members.fetch = async (id) => {
    if (typeof id === "object") {
      id = id.user;
    }
    const member = guild.members.cache.get(id);
    if (member)
      return Promise.resolve(member);
    return Promise.reject(new Error("Member not found"));
  };
  return guild;
}
__name(mockGuild, "mockGuild");
function mockRole(client, permissions, guild, role = {}) {
  if (!guild) {
    guild = mockGuild(client);
  }
  const roleData = {
    color: 0,
    hoist: false,
    id: randomSnowflake2().toString(),
    managed: false,
    mentionable: false,
    name: "test",
    position: 0,
    permissions: PermissionsBitField2.resolve(permissions).toString(),
    ...role
  };
  const createdRole = Reflect.construct(Role, [
    client,
    roleData,
    guild
  ]);
  guild.roles.cache.set(createdRole.id, createdRole);
  return createdRole;
}
__name(mockRole, "mockRole");

// src/message-mock.ts
import { Message, MessageType, MessagePayload, Embed, ActionRow, GuildEmoji, ReactionEmoji } from "discord.js";
import { randomSnowflake as randomSnowflake3 } from "@answeroverflow/discordjs-utils";
function mockEmbed(data) {
  return Reflect.construct(Embed, [
    data
  ]);
}
__name(mockEmbed, "mockEmbed");
function mockActionRow(data) {
  return Reflect.construct(ActionRow, [
    data
  ]);
}
__name(mockActionRow, "mockActionRow");
function applyMessagePayload(payload, message) {
  var _a, _b;
  if (typeof payload === "string") {
    message.content = payload;
  }
  if (payload instanceof MessagePayload) {
    throw new Error("Not implemented");
  }
  if (typeof payload !== "string") {
    message.embeds = ((_a = payload.embeds) == null ? void 0 : _a.map(mockEmbed)) ?? message.embeds;
    message.content = payload.content ?? message.content;
    message.components = ((_b = payload.components) == null ? void 0 : _b.map((comp) => mockActionRow(comp))) ?? message.components;
  }
  return message;
}
__name(applyMessagePayload, "applyMessagePayload");
function mockMessage(input) {
  const { client, override = {} } = input;
  let { author, channel } = input;
  if (!channel) {
    channel = mockTextChannel(client);
  }
  if (!author) {
    author = mockUser(client);
    if (!channel.isDMBased()) {
      mockGuildMember({
        client,
        user: author,
        guild: channel.guild
      });
    }
  }
  const rawData = {
    id: randomSnowflake3().toString(),
    channel_id: channel.id,
    author: {
      // TODO: Use a helper function to get properties
      id: author.id,
      username: author.username,
      discriminator: author.discriminator,
      avatar: author.avatar
    },
    content: "",
    timestamp: "",
    edited_timestamp: null,
    tts: false,
    mention_everyone: false,
    mentions: [],
    mention_roles: [],
    attachments: [],
    embeds: [],
    pinned: false,
    type: MessageType.Default,
    reactions: [],
    ...override
  };
  const message = Reflect.construct(Message, [
    client,
    rawData
  ]);
  channel.messages.cache.set(message.id, message);
  message.react = async (emoji) => {
    const isCustomEmoji = typeof emoji === "string" && emoji.startsWith("<:");
    if (emoji instanceof GuildEmoji) {
      throw new Error("Not implement");
    }
    if (emoji instanceof ReactionEmoji) {
      throw new Error("Not implement");
    }
    return Promise.resolve(mockReaction({
      message,
      user: client.user,
      override: {
        emoji: {
          id: isCustomEmoji ? emoji : null,
          name: isCustomEmoji ? null : emoji
        }
      }
    }));
  };
  message.startThread = async (options) => Promise.resolve(mockThreadFromParentMessage({
    client,
    parentMessage: message,
    data: options
  }));
  message.edit = (payload) => {
    return Promise.resolve(applyMessagePayload(payload, message));
  };
  message.delete = () => {
    channel == null ? void 0 : channel.messages.cache.delete(message.id);
    return Promise.resolve(message);
  };
  return message;
}
__name(mockMessage, "mockMessage");

// src/channel-mock.ts
function getGuildTextChannelMockDataBase(type, guild) {
  const rawData = {
    id: randomSnowflake4().toString(),
    type,
    position: 0,
    default_auto_archive_duration: 60,
    rate_limit_per_user: 0,
    flags: 0,
    guild_id: guild == null ? void 0 : guild.id,
    last_message_id: null,
    last_pin_timestamp: null,
    name: "channel name",
    nsfw: false,
    parent_id: null,
    permission_overwrites: [],
    topic: "channel topic"
  };
  return rawData;
}
__name(getGuildTextChannelMockDataBase, "getGuildTextChannelMockDataBase");
function setupMockedChannel(client, guild, createMockData) {
  if (!guild) {
    guild = mockGuild(client);
  }
  const channel = createMockData(guild);
  if (channel.type === ChannelType.GuildText || channel.type === ChannelType.GuildAnnouncement || channel.type === ChannelType.GuildForum) {
    channel.threads.fetchActive = () => {
      const activeThreads = [
        ...channel.threads.cache.values()
      ].filter((thread) => !thread.archived || thread.archived == null);
      const output = {
        threads: new Collection(activeThreads.map((thread) => [
          thread.id,
          thread
        ])),
        hasMore: false
      };
      return Promise.resolve(output);
    };
    channel.threads.fetchArchived = jest.fn().mockImplementation((options = {}) => {
      let filteredThreads = [
        ...channel.threads.cache.sorted((a, b) => isSnowflakeLargerAsInt(a.id, b.id)).values()
      ].filter((thread) => thread.archived);
      if (options.type) {
        filteredThreads = filteredThreads.filter((thread) => {
          const isPublicThread = thread.type === ChannelType.PublicThread;
          const isPrivateThread = thread.type === ChannelType.PrivateThread;
          if (options.type === "public" && isPublicThread) {
            return true;
          }
          if (options.type === "private" && isPrivateThread) {
            return true;
          }
          return false;
        });
      }
      filteredThreads = filteredThreads.filter((thread) => {
        let beforeTime = options.before;
        if (beforeTime == void 0) {
          return true;
        }
        if (beforeTime instanceof Date || typeof beforeTime === "number") {
          beforeTime = SnowflakeUtil.generate({
            timestamp: beforeTime
          }).toString();
        }
        if (beforeTime instanceof ThreadChannel) {
          beforeTime = beforeTime.id;
        }
        return isSnowflakeLarger(thread.id, beforeTime);
      });
      if (options.limit) {
        filteredThreads = filteredThreads.slice(0, options.limit);
      }
      const output = {
        threads: new Collection(filteredThreads.map((thread) => [
          thread.id,
          thread
        ])),
        hasMore: false
      };
      return Promise.resolve(output);
    });
    channel.createInvite = () => {
      const invite = mockInvite(client, channel);
      return Promise.resolve(invite);
    };
  }
  if (channel.isTextBased()) {
    channel.send = (options) => {
      const isMessagePayload = options instanceof MessagePayload2;
      const opts = isMessagePayload ? options.options : options;
      const message = mockMessage({
        client,
        channel,
        author: client.user,
        override: {
          content: typeof opts === "string" ? opts : opts.content ?? ""
        }
      });
      return Promise.resolve(message);
    };
    channel.messages.fetch = jest.fn().mockImplementation((q = {}) => {
      if (q instanceof Message2) {
        return Promise.resolve(q);
      }
      if (typeof q === "string") {
        const message = channel.messages.cache.get(q);
        if (!message) {
          return Promise.reject(new DiscordAPIError({
            code: 10008,
            message: "NOT FOUND"
          }, 10008, 404, "GET", "/channels/123/messages/123", {
            files: []
          }));
        }
        return Promise.resolve(message);
      }
      if ("message" in q) {
        const resolvable = q.message;
        if (resolvable instanceof Message2) {
          return Promise.resolve(q);
        }
        if (typeof resolvable === "string") {
          const message = channel.messages.cache.get(resolvable);
          if (!message) {
            return Promise.reject(new DiscordAPIError({
              code: 10008,
              message: "NOT FOUND"
            }, 10008, 404, "GET", "/channels/123/messages/123", {
              files: []
            }));
          }
          return Promise.resolve(message);
        }
        throw new Error("Invalid message");
      }
      const { after } = q;
      let { limit } = q;
      if (!limit) {
        limit = 100;
      }
      const sortedCachedMessages = sortMessagesById(Array.from(channel.messages.cache.values()));
      const filteredMessages = sortedCachedMessages.filter((message) => {
        if (!after)
          return true;
        return isSnowflakeLarger(message.id, after);
      });
      const messages = filteredMessages.slice(0, limit);
      jest.clearAllMocks();
      const asCollection = new Collection(messages.map((message) => [
        message.id,
        message
      ]));
      return Promise.resolve(asCollection);
    });
  }
  client.channels.cache.set(channel.id, channel);
  guild.channels.cache.set(channel.id, channel);
  return channel;
}
__name(setupMockedChannel, "setupMockedChannel");
function mockTextChannel(client, guild, data = {}) {
  return setupMockedChannel(client, guild, (guild2) => {
    const rawData = {
      ...getGuildTextChannelMockDataBase(ChannelType.GuildText, guild2),
      ...data
    };
    const channel = Reflect.construct(TextChannel, [
      guild2,
      rawData,
      client
    ]);
    return channel;
  });
}
__name(mockTextChannel, "mockTextChannel");
function mockThreadFromParentMessage(input) {
  const { client, parentMessage, data = {} } = input;
  if (parentMessage) {
    if (parentMessage && (parentMessage.channel.type === ChannelType.GuildText || parentMessage.channel.type === ChannelType.GuildAnnouncement)) {
      return mockPublicThread({
        client,
        parentChannel: parentMessage.channel,
        data: {
          id: parentMessage.id,
          ...data
        }
      });
    }
  }
  throw new Error("Invalid parent message");
}
__name(mockThreadFromParentMessage, "mockThreadFromParentMessage");
function mockPublicThread(input) {
  const { client, data = {} } = input;
  let { parentChannel } = input;
  return setupMockedChannel(client, parentChannel == null ? void 0 : parentChannel.guild, (guild) => {
    if (!parentChannel) {
      parentChannel = mockTextChannel(client, guild);
    }
    const rawData = {
      ...getGuildTextChannelMockDataBase(ChannelType.PublicThread, guild),
      member: {
        id: randomSnowflake4().toString(),
        user_id: randomSnowflake4().toString(),
        join_timestamp: (/* @__PURE__ */ new Date()).toISOString(),
        flags: 0
      },
      guild_id: guild.id,
      parent_id: parentChannel.id,
      applied_tags: [],
      message_count: 0,
      member_count: 0,
      thread_metadata: {
        archived: false,
        auto_archive_duration: 60,
        archive_timestamp: (/* @__PURE__ */ new Date()).toISOString(),
        locked: false
      },
      ...data
    };
    const thread = Reflect.construct(ThreadChannel, [
      guild,
      rawData,
      client
    ]);
    parentChannel.threads.cache.set(thread.id, thread);
    return thread;
  });
}
__name(mockPublicThread, "mockPublicThread");
function mockForumChannel(client, guild, data = {}) {
  return setupMockedChannel(client, guild, (guild2) => {
    const rawData = {
      ...getGuildTextChannelMockDataBase(ChannelType.GuildForum, guild2),
      available_tags: [
        {
          id: randomSnowflake4().toString(),
          name: "test tag",
          emoji_id: null,
          emoji_name: null,
          moderated: false
        }
      ],
      default_reaction_emoji: null,
      default_sort_order: null,
      ...data
    };
    const channel = Reflect.construct(ForumChannel, [
      guild2,
      rawData,
      client
    ]);
    return channel;
  });
}
__name(mockForumChannel, "mockForumChannel");
function mockNewsChannel(input) {
  return setupMockedChannel(input.client, input.guild, (guild) => {
    const rawData = {
      ...getGuildTextChannelMockDataBase(ChannelType.GuildAnnouncement, guild),
      ...input.data
    };
    const channel = Reflect.construct(NewsChannel, [
      guild,
      rawData,
      input.client
    ]);
    return channel;
  });
}
__name(mockNewsChannel, "mockNewsChannel");
function mockMessages(channel, numberOfMessages) {
  const messages = [];
  for (let id = 1; id <= numberOfMessages; id++) {
    messages.push(mockMessage({
      client: channel.client,
      channel
    }));
  }
  return messages;
}
__name(mockMessages, "mockMessages");
function mockMessageReaction({ message, reacter, override }) {
  var _a, _b;
  const data = {
    channel_id: message.channel.id,
    count: 1,
    emoji: {
      id: randomSnowflake4().toString(),
      name: "\u{1F44D}"
    },
    user_id: reacter.id,
    message_id: message.id,
    guild_id: (_a = message.guild) == null ? void 0 : _a.id,
    me: reacter.id === ((_b = message.client.user) == null ? void 0 : _b.id),
    ...override
  };
  const messageReaction = Reflect.construct(MessageReaction, [
    message.client,
    data,
    message
  ]);
  return messageReaction;
}
__name(mockMessageReaction, "mockMessageReaction");
function mockReaction({ message, user, override }) {
  var _a, _b;
  const data = {
    channel_id: message.channel.id,
    message_id: message.id,
    user_id: user.id,
    guild_id: (_a = message.guild) == null ? void 0 : _a.id,
    count: 1,
    emoji: {
      id: randomSnowflake4().toString(),
      name: "\u{1F44D}"
    },
    me: user.id === ((_b = message.client.user) == null ? void 0 : _b.id),
    ...override
  };
  const reaction = Reflect.construct(MessageReaction, [
    message.client,
    data,
    message
  ]);
  const emojiId = data.emoji.name ?? data.emoji.id;
  if (!emojiId) {
    throw new Error("Emoji ID and name cannot be null");
  }
  message.reactions.cache.set(emojiId, reaction);
  return reaction;
}
__name(mockReaction, "mockReaction");
function mockMarkedAsSolvedReply({ client, questionId, solutionId, channel, override = {} }) {
  const markedAsSolvedReply = mockMessage({
    client,
    author: client.user,
    override: {
      embeds: [
        {
          fields: [
            {
              name: "Solution Message ID",
              value: solutionId,
              inline: true
            },
            {
              name: "Question Message ID",
              value: questionId,
              inline: true
            }
          ]
        }
      ],
      ...override
    },
    channel
  });
  return markedAsSolvedReply;
}
__name(mockMarkedAsSolvedReply, "mockMarkedAsSolvedReply");
function mockInvite(client, channel, override = {}) {
  if (!channel) {
    channel = mockTextChannel(client);
  }
  const inviteData = {
    // random 5 letter string
    code: Math.random().toString(36).substring(2, 7),
    channel: {
      id: channel.id,
      name: channel.name,
      type: channel.type
    },
    ...override
  };
  const invite = Reflect.construct(Invite, [
    client,
    inviteData
  ]);
  return invite;
}
__name(mockInvite, "mockInvite");

// src/client-mock.ts
import { Client, Options } from "discord.js";
async function setupBot(override = {}) {
  const client = mockClient(override);
  await client.login();
  return client;
}
__name(setupBot, "setupBot");
function mockClient(override = {
  // Cache everything is used to simulate API responses, removes the limit
  makeCache: Options.cacheEverything()
}) {
  const client = new Client({
    intents: [],
    ...override
  });
  applyClientMocks(client);
  return client;
}
__name(mockClient, "mockClient");
function applyClientMocks(client) {
  Client.prototype.login = async () => Promise.resolve("");
  mockClientUser(client);
}
__name(applyClientMocks, "applyClientMocks");

// src/interaction-mock.ts
import { ChatInputCommandInteraction, InteractionType, ApplicationCommandType, PermissionsBitField as PermissionsBitField3, ButtonInteraction, ComponentType, InteractionResponse } from "discord.js";
import { randomSnowflake as randomSnowflake5 } from "@answeroverflow/discordjs-utils";

// src/to-api-data.ts
import { ChannelType as ChannelType2 } from "discord.js";
function userToAPIUser(user) {
  var _a;
  return {
    id: user.id,
    username: user.username,
    discriminator: user.discriminator,
    avatar: user.avatar,
    accent_color: user.accentColor,
    banner: user.banner,
    bot: user.bot,
    flags: (_a = user.flags) == null ? void 0 : _a.bitfield,
    system: user.system
  };
}
__name(userToAPIUser, "userToAPIUser");
function channelToAPIChannelMention(channel) {
  if (channel.isDMBased()) {
    throw new Error("Cannot mention a DM channel");
  }
  return {
    id: channel.id,
    guild_id: channel.guild.id,
    name: channel.name,
    type: channel.type
  };
}
__name(channelToAPIChannelMention, "channelToAPIChannelMention");
function messageToAPIData(message) {
  var _a, _b;
  return {
    id: message.id,
    attachments: [],
    author: userToAPIUser(message.author),
    channel_id: message.channel.id,
    content: message.content,
    edited_timestamp: ((_a = message.editedAt) == null ? void 0 : _a.toISOString()) ?? null,
    embeds: [],
    flags: (_b = message.flags) == null ? void 0 : _b.bitfield,
    mention_everyone: message.mentions.everyone,
    mention_roles: message.mentions.roles.map((role) => role.id),
    mentions: message.mentions.users.map((user) => userToAPIUser(user)),
    pinned: message.pinned,
    timestamp: message.createdAt.toISOString(),
    tts: message.tts,
    type: message.type,
    mention_channels: message.mentions.channels.map(channelToAPIChannelMention),
    nonce: message.nonce ?? void 0,
    position: message.position ?? void 0
  };
}
__name(messageToAPIData, "messageToAPIData");

// src/interaction-mock.ts
function mockChatInputCommandInteraction(client, name, id, guild, channel, member) {
  var _a, _b;
  if (!guild) {
    guild = mockGuild(client);
  }
  if (!channel) {
    channel = mockTextChannel(client, guild);
  }
  if (!member) {
    member = mockGuildMember({
      client,
      guild
    });
  }
  const rawData = {
    application_id: ((_a = client.user) == null ? void 0 : _a.id.toString()) ?? randomSnowflake5.toString(),
    channel_id: channel.id,
    id,
    type: InteractionType.ApplicationCommand,
    token: "123456789",
    version: 1,
    app_permissions: PermissionsBitField3.Default.toString(),
    locale: "en-US",
    guild_id: guild.id,
    member: {
      user: {
        id: member.id,
        username: member.user.username,
        discriminator: member.user.discriminator,
        avatar: member.user.avatar
      },
      roles: member.roles.cache.map((role) => role.id),
      premium_since: null,
      permissions: member.permissions.bitfield.toString(),
      pending: false,
      nick: member.nickname,
      mute: false,
      joined_at: ((_b = member.joinedAt) == null ? void 0 : _b.toISOString()) ?? (/* @__PURE__ */ new Date()).toISOString(),
      deaf: false
    },
    data: {
      id,
      name,
      type: ApplicationCommandType.ChatInput,
      guild_id: guild.id
    }
  };
  const command = Reflect.construct(ChatInputCommandInteraction, [
    client,
    rawData
  ]);
  command.reply = jest.fn();
  return command;
}
__name(mockChatInputCommandInteraction, "mockChatInputCommandInteraction");
function mockInteractionResponse({ interaction, id }) {
  return Reflect.construct(InteractionResponse, [
    interaction,
    id
  ]);
}
__name(mockInteractionResponse, "mockInteractionResponse");
function mockButtonInteraction({ client, override = {} }) {
  var _a, _b;
  const guild = mockGuild(client);
  const member = mockGuildMember({
    client,
    guild
  });
  const channel = mockTextChannel(client, guild);
  const message = mockMessage({
    client,
    channel,
    author: client.user
  });
  const id = randomSnowflake5.toString();
  const customId = override.custom_id ?? randomSnowflake5.toString();
  const rawData = {
    component_type: ComponentType.Button,
    application_id: ((_a = client.user) == null ? void 0 : _a.id.toString()) ?? randomSnowflake5.toString(),
    channel_id: channel.id,
    id,
    token: randomSnowflake5.toString(),
    version: 1,
    app_permissions: PermissionsBitField3.Default.toString(),
    locale: "en-US",
    guild_id: guild.id,
    member: {
      user: {
        id: member.id,
        username: member.user.username,
        discriminator: member.user.discriminator,
        avatar: member.user.avatar
      },
      roles: member.roles.cache.map((role) => role.id),
      premium_since: null,
      permissions: member.permissions.bitfield.toString(),
      pending: false,
      nick: member.nickname,
      mute: false,
      joined_at: ((_b = member.joinedAt) == null ? void 0 : _b.toISOString()) ?? (/* @__PURE__ */ new Date()).toISOString(),
      deaf: false
    },
    data: {
      component_type: ComponentType.Button,
      custom_id: customId
    },
    custom_id: customId,
    message: messageToAPIData(message),
    type: InteractionType.MessageComponent,
    ...override
  };
  const interaction = Reflect.construct(ButtonInteraction, [
    client,
    rawData
  ]);
  interaction.update = async (options) => {
    interaction.deferred = false;
    await interaction.message.edit(options);
    if (options instanceof Object && "fetchReply" in options) {
      return interaction.message;
    }
    return mockInteractionResponse({
      interaction,
      id: interaction.id
    });
  };
  interaction.deferUpdate = (options) => {
    interaction.deferred = true;
    if (options == null ? void 0 : options.fetchReply) {
      return Promise.resolve(interaction.message);
    }
    return Promise.resolve(mockInteractionResponse({
      id: interaction.id,
      interaction
    }));
  };
  interaction.reply = (opts) => {
    const msg = mockMessage({
      client,
      channel,
      author: client.user
    });
    interaction.deferred = false;
    interaction.replied = true;
    applyMessagePayload(opts, msg);
    if (opts instanceof Object && "fetchReply" in opts) {
      return Promise.resolve(msg);
    }
    return mockInteractionResponse({
      interaction,
      id: interaction.id
    });
  };
  interaction.editReply = (opts) => {
    interaction.deferred = false;
    interaction.replied = true;
    return interaction.message.edit(opts);
  };
  return interaction;
}
__name(mockButtonInteraction, "mockButtonInteraction");

// src/helpers.ts
import { PermissionFlagsBits } from "discord.js";
async function delay(timeInMs) {
  if (!timeInMs)
    timeInMs = process.env.DEFAULT_DELAY_IN_MS ? parseInt(process.env.DEFAULT_DELAY_IN_MS) : 500;
  await new Promise((resolve) => setTimeout(resolve, timeInMs));
}
__name(delay, "delay");
async function emitEvent(client, event, ...args) {
  const status = client.emit(event, ...args);
  await delay();
  return status;
}
__name(emitEvent, "emitEvent");
function overrideVariables(obj, overrides) {
  Object.assign(obj, overrides);
}
__name(overrideVariables, "overrideVariables");
function copyClass(obj, client, overrides = {}) {
  const created = Object.assign(
    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
    Object.create(Object.getPrototypeOf(obj)),
    obj
  );
  overrideVariables(created, {
    client,
    ...overrides
  });
  return created;
}
__name(copyClass, "copyClass");
async function testAllPermissions({ permissionsThatShouldWork, operation }) {
  for await (const permission of Object.keys(PermissionFlagsBits)) {
    const permissionIsAllowed = permissionsThatShouldWork.includes(permission);
    await operation(permission, permissionIsAllowed);
  }
}
__name(testAllPermissions, "testAllPermissions");

// src/scenarios.ts
import { PermissionFlagsBits as PermissionFlagsBits2 } from "discord.js";
async function createGuildMemberVariants(client, guild = void 0) {
  if (!guild)
    guild = mockGuild(client);
  const guildMemberOwner = await guild.members.fetch(guild.ownerId);
  const guildMemberDefault = mockGuildMember({
    client,
    guild
  });
  const pendingGuildMemberDefault = mockGuildMember({
    client,
    guild,
    data: {
      pending: true
    }
  });
  const guildMemberManageGuild = mockGuildMember({
    client,
    guild,
    permissions: PermissionFlagsBits2.ManageGuild
  });
  const guildMemberAdmin = mockGuildMember({
    client,
    guild,
    permissions: PermissionFlagsBits2.Administrator
  });
  return {
    guildMemberOwner,
    guildMemberDefault,
    pendingGuildMemberDefault,
    guildMemberManageGuild,
    guildMemberAdmin
  };
}
__name(createGuildMemberVariants, "createGuildMemberVariants");
export {
  applyClientMocks,
  copyClass,
  createGuildMemberVariants,
  delay,
  emitEvent,
  getGuildTextChannelMockDataBase,
  mockButtonInteraction,
  mockChatInputCommandInteraction,
  mockClient,
  mockClientUser,
  mockForumChannel,
  mockGuild,
  mockGuildMember,
  mockInteractionResponse,
  mockInvite,
  mockMarkedAsSolvedReply,
  mockMessageReaction,
  mockMessages,
  mockNewsChannel,
  mockPublicThread,
  mockReaction,
  mockRole,
  mockTextChannel,
  mockThreadFromParentMessage,
  mockUser,
  overrideVariables,
  setupBot,
  testAllPermissions
};
//# sourceMappingURL=index.js.map